#!/usr/bin/perl
#
#  Copyright 2007  Vladimir Voronin <vl@nchelny.ru>
#
# gen_pin_code_sql.pl 0.1
#

use DBI;
use Getopt::Std;

if (scalar(@ARGV) == 0) {
        usage();
}

my %option = ();
getopts("i:r:c:t:d:m:a:n:b:l:p:e:h", \%option);

if (defined($option{'h'})) {
        usage();
}

my $id_batch = '';

if (defined($option{'i'})) {
        $id_batch = $option{'i'};
	$id_batch =~ s/\s+//g;
}

my $realm = '';

if (defined($option{'r'}) && $option{'r'} =~ /^(\d{3})$/) {
        $realm = $1;
}

if ($realm eq '') {
	usage();
}

my $count_pin = 0;

if (defined($option{'c'})) {
	$count_pin = int($option{'c'});
}

if ($count_pin < 1) {
	usage();
}

my $tariff_plan = 'standard';

if (defined($option{'t'})) {
        $tariff_plan = $option{'t'};
}

my $date_stop = 'Dec 12 2099';

if (defined($option{'d'})) {
        if ($option{'d'} =~ /(\w{3}\s\d{2}\s\d{4})/) {
		$date_stop = $1;
	} else {
		usage();
	}
}

my $count_mon = 100;

if (defined($option{'m'})) {
        $count_mon = $option{'m'};
	if ($count_mon < 1) {
		usage();
	}
}

my $start_amount = 5;

if (defined($option{'a'})) {
        $start_amount = $option{'a'};
	if ($start_amount < 0) {
		usage();
	}
}

my $batch_desc = '';

if (defined($option{'n'})) {
        $batch_desc = $option{'n'};
}

my $billing = 1;

if (defined($option{'b'}) && $option{'b'} == 0) {
        $billing = 0;
}

my $pin_pattern = '#########';

if (defined($option{'l'})) {
        $pin_pattern = $option{'l'};
}

my $pass_pattern = '#';

if (defined($option{'p'})) {
        $pass_pattern = $option{'p'};
}

my $active = 0;

if (defined($option{'e'}) && $option{'e'} == 1) {
        $active = 1;
}

srand(time());

my $count = 0;
my %pins = ();

require '/etc/raddb/pcp.cfg';

if (!($db = DBI->connect( "dbi:$dbase_type:$database:$database_host", $dblogin, $dbpassword ))) {die "Error1";}

my $sql;
my $sth;

$sql = "select max(pin_id) from pins;";
$sth = $db->prepare($sql);
if (!$sth) {
	print STDERR "Error prepare query $sql\n";
	$db->disconnect();
	exit;
}

if (!$sth->execute()) {
	print STDERR "Error execute query $sql\n";
	$db->disconnect();
	exit;
}

my ($start) = $sth->fetchrow_array();
$sth->finish();
$start++;
sql_do($db, "insert into batches (batch_id,batch_desc,pin_pattern,password_pattern) values ('$id_batch','$batch_desc','$pin_pattern','$pass_pattern');");

while ($count < $count_pin) {
	my $pin_num = gen_pin_pattern($pin_pattern);
	next if (defined($pins{$pin_num}));
	$sql = "insert into pins (pin_number,pin_id,realm,status) values ('$pin_num',$pin_id,'$realm','i');";
	next if (!$db->do($sql));

	my $pass = gen_pin_pattern($pass_pattern);
	$pins{$p} = $pass;

	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'301','$start_amount');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'309','$billing');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'321','$start_amount');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'322','$tariff_plan');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'323','$id_batch');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'1002','$active');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'1005','$pass');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'1034','$date_stop');");
	sql_do($db, "insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'3006','$count_mon');");
	
	$count++;
	$start++;
}

# pattern
sub gen_pin_pattern ($) {
	my $s = shift;

	my $p = '';

	my @a = split(//, $s);
	foreach my $i (@a) {
		if ($i eq '#') {
			$p .= int(rand(10));
		} else {
			$p .= $i;
		}
	}
	return $p;
}

sub sql_do ($$)
{
	my $db = shift;
	my $sql = shift;

	if (!$db->do($sql)) {
		print STDERR "Error execute query $sql\n";
		$db->disconnect();
		exit;
	}
}

sub usage {
	print "        \n";
	print "Usage:\n";
	print "\t-h \t\thelp\n";
	print "\t-i <id batch>\tid ,       \n";
	print "\t-r <number>\t , 3 \n";
	print "\t-c <number>\t   \n";
	print "\t-t <plan>\t  ,   'standard'\n";
	print "\t-d <date>\t      'Mon DD YYYY'\n";
	print "\t\t\t 'Dec 01 2012'   'Dec 12 2099'\n";
	print "\t-m <num>\t        default 100\n";
	print "\t-a <num>\t    default 5\n";
	print "\t-n <string>\t  default ''\n";
	print "\t-b <0|1>\t   1   0 default 1\n";
	print "\t-l <>\t , default #########\n";
	print "\t-p <>\t , default #\n";
	print "\t-e <0|1>\t, 0 - , 1 -  default 0\n";
	print "\n\t  - 0-9,#   #   \n";
	exit;
}
