#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#  
#  Copyright 2002  The FreeRADIUS server project
#  Copyright 2002  Boian Jordanov <bjordanov@orbitel.bg>
#  Copyright 2005  Vladimir Voronin <vl@nchelny.ru>
#  

#
# perl-pcp 0.0.8
#

use strict;
# use ...
# This is very important ! Without this script will not get the filled hashesh from main.
use vars qw(%RAD_REQUEST %RAD_REPLY %RAD_CHECK);
use Data::Dumper;
use DBI;
use POSIX;

# This is hash wich hold original request from radius
#my %RAD_REQUEST;
# In this hash you add values that will be returned to NAS.
#my %RAD_REPLY;
#This is for check items
#my %RAD_CHECK;

my $dbh;
my $dbh_connect = 0;
my $g_log = 1;
my $DEBUG = 0;
my %parameters = ();
my @g_time_plan = ();
my %default_tariffs = ();

my $g_inp = '810';
my $g_np = '8';
my $g_lp = '9';

my %g_tables = ('Default-International-Zone' => 'countries', 'Default-National-Zone' => 'cities');
my %g_fields = ('Default-International-Zone' => 'country', 'Default-National-Zone' => 'city');
my %g_code_length = ('countries' => 'Country-Code-Length', 'cities' => 'City-Code-Length');

my %mons = (
	'Jan' => 0,
	'Feb' => 1,
	'Mar' => 2,
	'Apr' => 3,
	'May' => 4,
	'Jun' => 5,
	'Jul' => 6,
	'Aug' => 7,
	'Sep' => 8,
	'Oct' => 9,
	'Nov' => 10,
	'Dec' => 11);

my %wdays = (
	'Sun' => 0,
	'Mon' => 1,
	'Tue' => 2,
	'Wed' => 3,
	'Thu' => 4,
	'Fri' => 5,
	'Sat' => 6);

my @g_mabbr = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );

#
# This the remaping of return values 
#
	use constant    RLM_MODULE_REJECT=>    0;#  /* immediately reject the request */
	use constant	RLM_MODULE_FAIL=>      1;#  /* module failed, don't reply */
	use constant	RLM_MODULE_OK=>        2;#  /* the module is OK, continue */
	use constant	RLM_MODULE_HANDLED=>   3;#  /* the module handled the request, so stop. */
	use constant	RLM_MODULE_INVALID=>   4;#  /* the module considers the request invalid. */
	use constant	RLM_MODULE_USERLOCK=>  5;#  /* reject the request (user is locked out) */
	use constant	RLM_MODULE_NOTFOUND=>  6;#  /* user not found */
	use constant	RLM_MODULE_NOOP=>      7;#  /* module succeeded without doing anything */
	use constant	RLM_MODULE_UPDATED=>   8;#  /* OK (pairs modified) */
	use constant	RLM_MODULE_NUMCODES=>  9;#  /* How many return codes there are */

# Function to handle instantiate
# work if rlm_perl patched
sub instantiate {
	&radiusd::radlog(0,"init.");
	&Init_db();
	&Init_gdata();
	return 0;
}

# Function to handle authorize
sub authorize {
	# For debugging purposes only
	&radiusd::radlog(0,"authorize.");
	&log_request_attributes;

	# Here's where your authorization code comes
	# You can call another function from here:

	return &authorize_pcp;
	return RLM_MODULE_OK;
}

# Function to handle authenticate
sub authenticate {
	# For debugging purposes only
#	&log_request_attributes;

#	return &authorize_pcp;
#	if ($RAD_REQUEST{'User-Name'} =~ /^baduser/i) {
		# Reject user and tell him why
#		$RAD_REPLY{'Reply-Message'} = "Denied access by rlm_perl function";
#		return RLM_MODULE_REJECT;
#	} else {
		# Accept user and set some attribute
#		$RAD_REPLY{'h323-credit-amount'} = "100";
		return RLM_MODULE_OK;
#	}
}

# Function to handle preacct
sub preacct {
	# For debugging purposes only
#	&log_request_attributes;
	
	return RLM_MODULE_OK;
}

# Function to handle accounting
sub accounting {
	# For debugging purposes only
#	&log_request_attributes;

	# You can call another subroutine from here 
	&account_pcp_stop;
	
	return RLM_MODULE_OK;
}

# Function to handle checksimul
sub checksimul {
	# For debugging purposes only
#	&log_request_attributes;

	return RLM_MODULE_OK;
}

# Function to handle xlat
sub xlat {
	# For debugging purposes only
#	&log_request_attributes;

	&radiusd::radlog(0, "XLAT ");
	# Loads some external perl and evaluate it
	my ($filename,$a,$b,$c,$d) = @_;
	&radiusd::radlog(1, "From xlat $filename ");
	&radiusd::radlog(1,"From xlat $a $b $c $d ");
	local *FH;
	open FH, $filename or die "open '$filename' $!";
	local($/) = undef;
	my $sub = <FH>;
	close FH;
	my $eval = qq{ sub handler{ $sub;} };
	eval $eval;
	eval {main->handler;};
}

# Function to handle detach
sub detach {
	# For debugging purposes only
#	&log_request_attributes;

	&Init_gdata();
	# Do some logging.
	&radiusd::radlog(0,"Detaching. Reloading. Done.");
}

#
# Some functions that can be called from other functions
#

sub authorize_pcp {
	# Some code goes here 
	my $max_length_code = 0;
	my $datetime;
	my $sql;

	&Init_db if ($dbh_connect == 0);
	#       perl     
	#        
	if (!defined($parameters{'Country-Code-Length'})) {
		&radiusd::radlog($DEBUG,"pcp::authorize_pcp() Error parameters not loaded!");
		foreach my $i (sort keys %parameters) {
			&radiusd::radlog($DEBUG, "PCP PARAMETER: $i = $parameters{$i}");
		}
		Init_gdata();
	}

	my %source = get_source($RAD_REQUEST{'NAS-IP-Address'});

	if (!defined($source{'zone'})) {
		&radiusd::radlog($DEBUG,"pcp::authorize_pcp() source ".$RAD_REQUEST{'NAS-IP-Address'}." not found");
		return RLM_MODULE_REJECT;
	}

	my ($Pin, $Realm) = get_pin_realm($RAD_REQUEST{'User-Name'});
	my %pins = get_pin_data($Pin, $Realm);

	if (!check_pin(\%pins)) {
		&radiusd::radlog($DEBUG,"pcp::authorize_pcp() check pin false");
#		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=3';
		return RLM_MODULE_REJECT;
	}

	my $pin_id = $pins{'pin_id'};

	if (defined($pins{'1005'})) {
		$datetime = get_curtime();

		if (!defined($RAD_REQUEST{'Called-Station-Id'})) {
			$sql = "INSERT INTO log_auth (user_id,client_id,pin_id,rc,realm,pin_number,op_date) VALUES('";
			$sql .= $RAD_REQUEST{'NAS-IP-Address'} . "','" . $RAD_REQUEST{'Calling-Station-Id'};
			$sql .= "','$pin_id','0','$Realm','$Pin','$datetime')";
			sql_do($sql, 'authorize_pcp()');
		}

		if (defined($RAD_REQUEST{'Calling-Station-Id'}) &&
		    length($RAD_REQUEST{'Calling-Station-Id'}) > 0 &&
		    !check_block_ani($RAD_REQUEST{'Calling-Station-Id'}, $pins{'322'}, $source{'id'})) {
			return RLM_MODULE_REJECT;
		}

		if (!check_pin_status($pins{'status'})) {
			&radiusd::radlog($DEBUG,"pcp::authorize_pcp() pin status for pin id ".$pins{'pin_id'}." is blocked");
			&radiusd::radlog($DEBUG,"pcp::authorize_pcp() h323-return-code=".$RAD_REPLY{'h323-return-code'});
			return RLM_MODULE_REJECT;
		}

		if ($pins{'301'} < 0) {
			$pins{'301'} = 0;
#			$RAD_REPLY{'h323-return-code'} = 'h323-return-code=4';
		}
		#     
		#    ,   h323-credit-amount
		$RAD_REPLY{'h323-credit-amount'} = 'h323-credit-amount='.sprintf("%.2f", $pins{'301'});

		if (check_pin_in_use($pin_id) != RLM_MODULE_OK) {
			&radiusd::radlog($DEBUG,"pcp::authorize_pcp() pin id ".$pins{'pin_id'}." in use");
			return RLM_MODULE_REJECT;
		}

		if (defined($RAD_REQUEST{'Called-Station-Id'}) && $pins{'301'} > 0) {
			my $credit_time = 0;

			#    ,   debitcard.tcl
			#        
			if ($pins{'301'} <= 0) {
				#  
				$RAD_REPLY{'h323-return-code'} = 'h323-return-code=4';
				&radiusd::radlog($DEBUG,"pcp::authorize_pcp() balance to pin id ".$pins{'pin_id'}." is 0 or < 0");
				return RLM_MODULE_REJECT;
			}

			my %pd = get_called_data($pins{'322'});

			#  
			if (defined($RAD_REPLY{'h323-return-code'})) {
				&radiusd::radlog($DEBUG,"pcp::authorize_pcp() ERROR pin id ".$pins{'pin_id'}." h323-return-code=".$RAD_REPLY{'h323-return-code'});
				return RLM_MODULE_REJECT
			}

			#        
			if (check_blocked_dest($pins{'pin_id'}, $RAD_REQUEST{'Called-Station-Id'})) {
				$RAD_REPLY{'h323-return-code'} = 'h323-return-code=9';
				&radiusd::radlog($DEBUG,"pcp::authorize_pcp() CalledId ".$RAD_REQUEST{'Called-Station-Id'}." blocked");
				return RLM_MODULE_REJECT
			}

			my ($tariff, $time_plan) = get_tariff($source{'zone'}, $pd{'dest_zone'}, $pins{'322'}, $pd{'def_zone'});
			$credit_time = int($pins{'301'} / $tariff) * $parameters{'Tariff-Base'} if ($tariff > 0);

			if ($credit_time == 0) {
				#     1 
				$RAD_REPLY{'h323-return-code'} = 'h323-return-code=12';
				&radiusd::radlog($DEBUG,"pcp::authorize_pcp() not many for call to pin id ".$pins{'pin_id'});
				return RLM_MODULE_REJECT;
			}

			$credit_time = $parameters{'Maximum-Talk-Time'} if ($credit_time > $parameters{'Maximum-Talk-Time'});
			$datetime = get_curtime();

			$sql = "INSERT INTO pin_in_use (realm,pin_id,pin_number,called_station_id,";
			$sql .= "source_ip,start_time,max_talk_time,src_zone,dest_zone,code,";
			$sql .= "plan_num,time_plan,tariff,source_node,conf_id,dest_name) VALUES('$Realm',";
			$sql .= "'$pin_id','$Pin','".$RAD_REQUEST{'Called-Station-Id'}."','";
			$sql .= $RAD_REQUEST{'NAS-IP-Address'}."','$datetime','$credit_time','".$source{'zone'}."',";
			$sql .= "'" . $pd{'dest_zone'} . "','".$pd{'code'}."','".$pins{'322'}."','$time_plan',";
			$sql .= "'$tariff','".$RAD_REQUEST{'NAS-IP-Address'}."','".$RAD_REQUEST{'h323-conf-id'}."',";
			$sql .= "'".$pd{'zone_name'}."');";

			del_pin_in_use($pins{'pin_id'});
			sql_do($sql, 'authorize_pcp()');

			$RAD_REPLY{'h323-credit-time'} = "h323-credit-time=$credit_time";
		}

		$RAD_REPLY{'Service-Type'} = 'Login';
		$pins{'309'} = 1 if (!defined($pins{'309'}));
		$RAD_REPLY{'h323-billing-model'} = 'h323-billing-model=' . $pins{'309'};
		if (!defined($RAD_REPLY{'h323-return-code'})) {
			&radiusd::radlog($DEBUG,"pcp::authorize_pcp() Login OK for pin id " . $pins{'pin_id'});
			$RAD_REPLY{'h323-return-code'} = 'h323-return-code=0';
		} else {
			&radiusd::radlog($DEBUG,"pcp::authorize_pcp() h323-return-code=".$RAD_REPLY{'h323-return-code'});
		}
		return RLM_MODULE_OK;
	}

	return RLM_MODULE_REJECT;
}

sub account_pcp_stop {
	#       perl     
	#        
	if (!defined($parameters{'Country-Code-Length'})) {
		&radiusd::radlog($DEBUG,"pcp::authorize_pcp() Error parameters not loaded!");
		foreach my $i (sort keys %parameters) {
			&radiusd::radlog($DEBUG, "PCP PARAMETER: $i = $parameters{$i}");
		}
		Init_gdata();
	}
	if ($RAD_REQUEST{'Acct-Status-Type'} eq 'Start' && $RAD_REQUEST{'h323-call-type'} eq 'VoIP') {
		&Init_db if ($dbh_connect == 0);
		my ($Pin,$Realm) = get_pin_realm($RAD_REQUEST{'User-Name'});
		my %pins = get_pin_data($Pin, $Realm);
		my $pin_id = $pins{'pin_id'};
		if (defined($pins{'pin_id'})) {
			set_status_in_use($pin_id);
		}
		return RLM_MODULE_OK;
	}
	if ($RAD_REQUEST{'Acct-Status-Type'} ne 'Stop') {
		return RLM_MODULE_OK;
	}

#	if ($RAD_REQUEST{'h323-call-type'} eq 'Telephony') {
#		return RLM_MODULE_OK;
#	}

#	if ($RAD_REQUEST{'h323-call-type'} ne 'VoIP') {
#		return RLM_MODULE_OK;
#	}

	&Init_db if ($dbh_connect == 0);
	my ($Pin,$Realm) = get_pin_realm($RAD_REQUEST{'User-Name'});
#	my %pins = check_pin_acc($Pin, $Realm);
	my %pins = get_pin_data($Pin, $Realm);
	my $pin_id = $pins{'pin_id'};

	if (defined($pins{'pin_id'}) && $RAD_REQUEST{'h323-call-type'} eq 'Telephony') {
		#       VoIP   
		#     
#		if (check_pin_in_use($pin_id) == RLM_MODULE_OK) {
			del_pin_in_use($pins{'pin_id'});
#		}
	} elsif (defined($pins{'pin_id'}) && $RAD_REQUEST{'h323-call-type'} eq 'VoIP') {
		#  cisco, ,   Called-Station-Id
		my $Called_Station_Id = '';
		if (ref($RAD_REQUEST{'Called-Station-Id'}) eq 'ARRAY') {
			$Called_Station_Id = $RAD_REQUEST{'Called-Station-Id'}[$#{ $RAD_REQUEST{'Called-Station-Id'} }];
		} else {
			$Called_Station_Id = $RAD_REQUEST{'Called-Station-Id'};
		}

		my $datetime = '';
		my %tu = get_inuse($pins{'pin_id'}, $RAD_REQUEST{'h323-conf-id'});
		my $tariff = $tu{'tariff'};
		$tu{'start_time'} = get_curtime() if ($tu{'start_time'} eq '');
#		if (!defined($tu{'src_zone'})) {
			my %source = get_source($RAD_REQUEST{'NAS-IP-Address'});
#			$tu{'src_zone'} = $source{'zone'};
#			$tu{'source_id'} = $source{'id'};
#		}
		if ($tariff == 0) {
			my %pd = get_called_data($pins{'322'});
			($tariff, $tu{'plan_num'}) = get_tariff($tu{'src_zone'}, $pd{'dest_zone'}, $pins{'322'}, $pd{'def_zone'});
		}
		my $sum = 0;
		my $sql;

&log_request_attributes;

		if ($RAD_REQUEST{'Acct-Session-Time'} > $parameters{'Not-Billed-Interval'}) {
			#   ,         
			$sum = sprintf("%.4f", ceil($RAD_REQUEST{'Acct-Session-Time'} / $parameters{'Tariff-Base'}) * $tariff);
			$sql = "UPDATE pin_attributes SET attr_val=attr_val-$sum WHERE attr_num=301 AND pin_id=$pin_id;";
			sql_do($sql, 'account_pcp_stop()');
		}

#		my %source = get_source($RAD_REQUEST{'NAS-IP-Address'});

#		if (!defined($source{'name'})) {
#			&radiusd::radlog(1,"pcp::account_pcp_stop() source ".$RAD_REQUEST{'NAS-IP-Address'}." not found");
#			$source{'name'} = '';
#		}

		$sql = "INSERT INTO billing (session_date,nas_identifier,nas_ip_address";
$sql .= ",service_type,acct_authentic,acct_input_octets,acct_output_octets,acct_session_id,acct_session_time";
$sql .= ",acct_input_packets,acct_output_packets,acct_delay_time,pin_number,realm,calling_station_id";
$sql .= ",called_station_id,code,cost,h323_rem_gate_id,h323_connection_id,h323_setup_time,h323_connect_time";
$sql .= ",h323_disconnect_time,h323_disconnect_cause,h323_voice_quality,password,batch_id,current_amount,plan_num";
$sql .= ",time_plan,pin_id,tariff,dest_zone,dest_name,src_zone,source_node,source_id,source_name,backend_type";
$sql .= ",round_session_time) VALUES (";
		$sql .= "'" . $tu{'start_time'} . "',''";
		$sql .= ",'" . $RAD_REQUEST{'NAS-IP-Address'} . "'";
		$sql .= ",'1','0','" . $RAD_REQUEST{'Acct-Input-Octets'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Output-Octets'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Session-Id'}."'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Session-Time'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Input-Packets'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Output-Packets'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'Acct-Delay-Time'} . "'";
		$sql .= ",'$Pin\@$Realm','$Realm'";
		$sql .= ",'" . $RAD_REQUEST{'Calling-Station-Id'} . "'";
#		$sql .= ",'" . get_called_st_id($RAD_REQUEST{'Called-Station-Id'}) . "'";
		$sql .= ",'" . $Called_Station_Id . "'";
#		$sql .= ",'$pnumber'";
		$sql .= ",'" . $tu{'code'} . "'";
		if ($sum > 0) {
			$sql .= ",'" . sprintf("%16s", sprintf("%.4f", $sum)) . "'";
		} else {
			$sql .= ",'" . sprintf("%16s", 0) . "'";
		}
		$sql .= ",'" . $RAD_REQUEST{'h323-remote-address'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-conf-id'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-setup-time'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-connect-time'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-disconnect-time'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-disconnect-cause'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'h323-voice-quality'} . "'";
		$sql .= ",'" . $pins{'1005'} . "'";
		$sql .= ",'" . $pins{'323'} . "'";
		$sql .= ",'" . sprintf("%.6f", $pins{'301'}) . "'";
		$sql .= ",'" . $pins{'322'} . "'";
#		$sql .= ",'" . get_time_plan(time() - $RAD_REQUEST{'Acct-Session-Time'}) . "'";
		$sql .= ",'" . $tu{'time_plan'} . "'";
		$sql .= ",'" . $pins{'pin_id'} . "'";
		$sql .= ",'" . sprintf("%.6f", $tariff) . "'";
		$sql .= ",'" . $tu{'dest_zone'} . "','" . $tu{'dest_name'} . "','" . $source{'zone'} . "'";
		$sql .= ",'" . $RAD_REQUEST{'NAS-IP-Address'} . "'";
		$sql .= ",'" . $source{'id'} . "'";
		$sql .= ",'" . $source{'name'} . "'";
		$sql .= ",'" . uc($RAD_REQUEST{'h323-call-type'}) . "'";
		if ($RAD_REQUEST{'Acct-Session-Time'} > $parameters{'Not-Billed-Interval'}) {
			$sum = ceil($RAD_REQUEST{'Acct-Session-Time'} / $parameters{'Tariff-Base'});
		} else {
			$sum = 0;
		}
		$sql .= ",'$sum'";
		$sql .= ");";
		sql_do($sql, 'account_pcp_stop()');

#		if (check_pin_in_use($pin_id) == RLM_MODULE_OK) {
&radiusd::radlog($DEBUG,"pcp::account_pcp_stop() DEBUG!!! DELETE FROM");
			del_pin_in_use($pins{'pin_id'}, 'i');
#		}
	}

	return RLM_MODULE_OK;
}

sub log_request_attributes {
	my $i;
	# This shouldn't be done in production environments!
	# This is only meant for debugging!
	for $i (keys %RAD_REQUEST) {
		&radiusd::radlog($DEBUG, "RAD_REQUEST: $i = " . $RAD_REQUEST{$i});
	}
}

sub Init_db {
	my $name_func = 'Init_gdata';
	my $dbase_type = 'mysql';
	my $database = 'pcp';
	my $database_host = 'localhost';
	my $dblogin = 'pcp';
	my $dbpassword = '';

	# set this data if mysql server > version 4.0
	my $character_set_client = 'koi8r';
	my $character_set_connection = 'koi8r';
	my $character_set_database = 'koi8r';
	my $character_set_results = 'koi8r';
	my $character_set_server = 'koi8r';

	my %options = load_config('/etc/raddb/pcp.cfg');

	$DEBUG = $options{'DEBUG'} if (defined($options{'DEBUG'}));

	$dbase_type = $options{'dbase_type'} if (defined($options{'dbase_type'}));
	$database = $options{'database'} if (defined($options{'database'}));
	$database_host = $options{'database_host'} if (defined($options{'database_host'}));
	$dblogin = $options{'dblogin'} if (defined($options{'dblogin'}));
	$dbpassword = $options{'dbpassword'} if (defined($options{'dbpassword'}));


	if ($dbh_connect) {
		$dbh->disconnect();
	}

	$dbh_connect = 0;

	if (($dbh = DBI->connect("dbi:$dbase_type:$database:$database_host", $dblogin, $dbpassword))) {
		$dbh_connect = 1;
		if ($dbase_type eq 'mysql' and length($character_set_client) > 0) {
			$character_set_client = $options{'character_set_client'} if (defined($options{'character_set_client'}));
			$character_set_connection = $options{'character_set_connection'} if (defined($options{'character_set_connection'}));
			$character_set_database = $options{'character_set_database'} if (defined($options{'character_set_database'}));
			$character_set_results = $options{'character_set_results'} if (defined($options{'character_set_results'}));
			$character_set_server = $options{'character_set_server'} if (defined($options{'character_set_server'}));

			sql_do("set character_set_client=$character_set_client;", $name_func);
			sql_do("set character_set_connection=$character_set_connection;", $name_func);
			sql_do("set character_set_database=$character_set_database;", $name_func);
			sql_do("set character_set_results=$character_set_results;", $name_func);
			sql_do("set character_set_server=$character_set_server;", $name_func);
		}
	} else {
		&radiusd::radlog(0, "pcp::Init_db() Error connect to $dbase_type host $database_host database $database with login $dblogin");
	}

	return $dbh_connect;
}

sub Init_gdata {
	if ($dbh_connect == 0 && !Init_db()) {
		return 0;
	}

	my $sql;
	my $sth;
	my $name_func = 'Init_gdata';

	# get parameters
	$sql = "select * from parameters;";
	$sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {

		if ($sth->rows > 0) {
			#       
			#      ,   
			%parameters = ();
			while(my $data = $sth->fetchrow_hashref) {
				$parameters{$data->{name}} = $data->{value};
			}
		}
		}

		$sth->finish();
	}

	$parameters{'City-Code-Length'} = 7 if (!defined($parameters{'City-Code-Length'}));
	$parameters{'Country-Code-Length'} = 7 if (!defined($parameters{'Country-Code-Length'}));
	$parameters{'Default-International-Code'} = 'DEF-I' if (!defined($parameters{'Default-International-Code'}));
	$parameters{'Default-International-Zone'} = '999' if (!defined($parameters{'Default-International-Zone'}));
	$parameters{'Default-National-Code'} = 'DEF-N' if (!defined($parameters{'Default-National-Code'}));
	$parameters{'Default-National-Zone'} = '99' if (!defined($parameters{'Default-National-Zone'}));
	$parameters{'International-Prefix-1'} = '1' if (!defined($parameters{'International-Prefix-1'}));
	$parameters{'International-Prefix-2'} = '0' if (!defined($parameters{'International-Prefix-2'}));
	$parameters{'International-Prefix-Length'} = 3 if (!defined($parameters{'International-Prefix-Length'}));
	$parameters{'Maximum-Talk-Time'} = 1800 if (!defined($parameters{'Maximum-Talk-Time'}));
	$parameters{'Min-National-Number-Length'} = 10 if (!defined($parameters{'Min-National-Number-Length'}));
	$parameters{'Max-National-Number-Length'} = 10 if (!defined($parameters{'Max-National-Number-Length'}));
	$parameters{'Not-Billed-Interval'} = 24 if (!defined($parameters{'Not-Billed-Interval'}));
	$parameters{'National-Prefix'} = '8' if (!defined($parameters{'National-Prefix'}));
	$parameters{'National-Prefix-Length'} = 1 if (!defined($parameters{'National-Prefix-Length'}));
	$parameters{'National-Code'} = '' if (!defined($parameters{'National-Code'}));
	$parameters{'Tariff-Base'} = 60 if (!defined($parameters{'Tariff-Base'}));

	$g_np = $parameters{'National-Prefix'};
	$g_inp = $parameters{'National-Prefix'} . $parameters{'International-Prefix-1'} . $parameters{'International-Prefix-2'};

	if ($g_log) {
		foreach my $i (sort keys %parameters) {
			&radiusd::radlog($DEBUG, "PCP PARAMETER: $i = $parameters{$i}");
		}
	}

	# get time plans
	$sql = "select time_plan,time_interval from times order by priority desc;";
	$sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
		if ($sth->rows > 0) {
			#       
			#      ,   
			@g_time_plan = ();
			my $count = 0;

			while(my ($plan,$int) = $sth->fetchrow_array) {
				$int =~ s/\t/ /g;
				$g_time_plan[$count] = "$plan\t$int";
				$count ++;
			}
		}
		}

		$sth->finish();
	}

	# default tariffs
	$sql = "select * from default_tariffs;";
	$sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
		if ($sth->rows > 0) {
			%default_tariffs = ();
		}
		while(my $data = $sth->fetchrow_hashref) {
			$default_tariffs{$data->{'src_zone'}."\t".$data->{'dest_zone'}."\t".$data->{'time_plan'}} = $data->{'tariff'};
		}
		}
		$sth->finish();
	}

	return 1;
}

sub get_pin_realm($) {
	my $User = shift;

	if ($User =~ /^\!(\d+)\@(\d+)/) {
		return ($1,$2);
	}

	return ('','');
}

sub check_pin_status($)
{
	my $status = shift;

	if ($status eq 'i') {
		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=7';
		return 0;
	}

	return 1;
}

# date in format 'Dec 01 2012'
sub check_date_to_cur($)
{
	my $date = shift;

	if ($date =~ /(\w{3})\s+(\d+)\s+(\d{4})/) {
		my ($mday,$mon,$year) = (localtime)[3,4,5];
		$year += 1900;
		return 1 if ($year < $3);
		return 0 if ($year > $3);
		return 1 if ($mon < $mons{$1});
		return 0 if ($mon > $mons{$1});
		return 0 if ($mday > $2);
	}

	return 1;
}

# point to hash pin data
sub check_pin($)
{
	my $p = shift;

	if (!defined($p->{'pin_id'})) {
		return 1;
	}

	my $pin_id = $p->{'pin_id'};

	# check for date expiration
	if (defined($p->{'1034'})) {
		if (!check_date_to_cur($p->{'1034'})) {
			$RAD_REPLY{'h323-return-code'} = 'h323-return-code=5';
			return 0;
		}
	}

	if (defined($p->{'3005'})) {
		if (!check_date_to_cur($p->{'3005'})) {
			$RAD_REPLY{'h323-return-code'} = 'h323-return-code=5';
			return 0;
		}
	} else {
		if (defined($p->{'3006'}) && $p->{'3006'} > 0) {
			my ($mday,$mon,$year) = (localtime(time() + $p->{'3006'} * 2678400))[3,4,5];
			$year += 1900;
			my $start = "$g_mabbr[$mon] $mday $year";
			sql_do("insert into pin_attributes (pin_id,attr_num,attr_val) values ($pin_id,'3005','$start');");
		}
	}

	return 1;
}

# pin
# realm
sub get_pin_data ($$) {
	my $Pin = shift;
	my $Realm = shift;

	my $name_func = 'get_pin_data';
	my %pins = ();
	my $sql = "SELECT /*+ first_rows*/ pins.pin_id,pin_number,realm,status,attr_num,0
,attr_val FROM pins,pin_attributes WHERE pins.pin_id = pin_attributes.pin_id  AND pin_number = '$Pin'
 AND realm = '$Realm' ORDER BY pin_id";

	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
	if (sql_execute($sth, $name_func)) {

	if ($sth->rows > 0) {
		while((my $data = $sth->fetchrow_hashref)) {
			$pins{'pin_id'} = $data->{pin_id};
			$pins{$data->{attr_num}} = $data->{attr_val};
			$pins{'status'} = $data->{status};
			&radiusd::radlog($DEBUG,"pcp::$name_func() ".$data->{attr_num}." = ".$data->{attr_val});
		}
		if (defined($RAD_REQUEST{'User-Password'})
		    && $pins{'1005'} != $RAD_REQUEST{'User-Password'}) {
			%pins = ();
		}
	}
	}
	$sth->finish();
	}

	if (!defined($pins{'pin_id'})) {
		&radiusd::radlog($DEBUG,"pcp::$name_func() pin_id for PIN=$Pin\@$Realm not found");
		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=1';
	}

	return %pins;
}

sub check_pin_in_use ($) {
	my $pin_id = shift;

	my $name_func = 'check_pin_in_use';
	my $ret = RLM_MODULE_OK;
	my $sql = "SELECT start_time,called_station_id,conf_id,status FROM pin_in_use WHERE pin_id=$pin_id";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
	if (sql_execute($sth, $name_func)) {

	if ($sth->rows > 0) {
		my $data = $sth->fetchrow_hashref;
		$sth->finish();

		if ($data->{conf_id} ne $RAD_REQUEST{'h323-conf-id'}) {
			$RAD_REPLY{'h323-return-code'} = 'h323-return-code=3';
&radiusd::radlog($DEBUG,"pcp::$name_func() PIN ID $pin_id in pin_in_use");
			$ret = RLM_MODULE_REJECT;
		}
	}
	}
	$sth->finish();
	}

	return $ret;
}

# ip address gateway
sub get_source($) {
	my $ip = shift;

	my $name_func = 'get_source';
	my %ret = ();
	my $sql = "SELECT source_zone,source_id,source_name FROM sources WHERE source_ip='$ip';";
	my $sth = sql_prepare($sql, $name_func);

	if ($sth) {
		if (sql_execute($sth, $name_func)) {

		if ($sth->rows > 0) {
			($ret{'zone'}, $ret{'id'}, $ret{'name'}) = $sth->fetchrow_array;
			&radiusd::radlog($DEBUG,"pcp::$name_func() Found sources $ip zone ".$ret{'zone'}." name ".$ret{'name'});
		}
		}

		$sth->finish();
	}

	return %ret;
}

# zone num
sub get_time_diff($) {
	my $zn = shift;

	my $name_func = 'get_time_diff';
	my $td = 0;
	my $sql = "SELECT time_dif FROM zones WHERE zone_num=$zn;";
	my $sth = sql_prepare($sql, $name_func);

	if ($sth) {
		if (sql_execute($sth, $name_func)) {
			if ($sth->rows > 0) {
				($td) = $sth->fetchrow_array;
			}
		}

		$sth->finish();
	}

	return $td;
}

# Called-Station-Id
sub cut_code_prefix ($) {
	my $pnumber = shift;

	my $def_zone = '';

	&radiusd::radlog($DEBUG,"pcp::cut_code_prefix() number = $pnumber International prefix = $g_inp National prefix = $g_np");

# FIXME
	if ($pnumber =~ /^$g_inp(\d+)/) {
		if (defined($RAD_REQUEST{'gw-collected-cdn'}) && $RAD_REQUEST{'gw-collected-cdn'} =~ /^$g_inp/) {
			$def_zone = 'Default-International-Zone';
		} else {
			$pnumber = $1;
			$def_zone = 'Default-International-Zone';
			$RAD_REPLY{'h323-redirect-number'} = "h323-redirect-number=$pnumber";
		}
	} elsif ($pnumber =~ /^$g_np(\d+)/) {
		$pnumber = $1;
		my $len = length($pnumber);
		if ($len >= $parameters{'Min-National-Number-Length'} &&
		    $len <= $parameters{'Max-National-Number-Length'}) {
			my $num = $parameters{'National-Code'} . $pnumber;
			$def_zone = 'Default-National-Zone';
			$RAD_REPLY{'h323-redirect-number'} = "h323-redirect-number=$num";
		} else {
	&radiusd::radlog($DEBUG,"pcp::cut_code_prefix() length $pnumber = ". length($pnumber));
		}
	}

	&radiusd::radlog($DEBUG,"pcp::cut_code_prefix() number = $pnumber, default zone = $def_zone " . $RAD_REPLY{'h323-redirect-number'});
	return ($pnumber, $def_zone);
}

# code without code country
# plan num
# name field
# name table
sub get_max_length_code ($$$$) {
	my $pnumber = shift;
	my $plan_num = shift;
	my $c = shift;
	my $cc = shift;

	my $name_func = 'get_max_length_code';
	my $count = 1;
	my $max_length_code = 0;

	if ($c eq '' || $cc eq '') {
		#    
		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=1';
		&radiusd::radlog($DEBUG,"pcp::$name_func() not defined table name or field name");
		return $max_length_code;
	}

	&radiusd::radlog($DEBUG,"pcp::get_max_length_code() table = $cc");

	my $sql = "SELECT max(length(".$c."_code)) FROM $cc WHERE plan_num='$plan_num'";
	$sql .= " AND ".$c."_code  IN (";

	while ($count <= $parameters{$g_code_length{$cc}}) {
		$sql .= "'" . substr($pnumber, 0, $count) . "'";
		$sql .= "," if ($count < $parameters{$g_code_length{$cc}});
		$count++;
	}

	$sql .= ");";

	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
			if ($sth->rows > 0) {
				($max_length_code) = $sth->fetchrow_array;
			}
		}

		$sth->finish();
	}

	if ($max_length_code == 0) {
		#    
		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=1';
		&radiusd::radlog($DEBUG,"pcp::get_max_length_code() not found code $pnumber from $cc");
	}

	return $max_length_code;
}

# code
# plan num
# name field
# name table
sub get_dest_zone($$$$) {
	my $code = shift;
	my $plan_num = shift;
	my $c = shift;
	my $cc = shift;

	my $name_func = 'get_dest_zone';
	my $dest_zone = 0;
	my $zone_name = '';

	my $sql = "SELECT ".$c."_zone,".$c."_name FROM $cc WHERE ".$c."_code='$code' AND plan_num='$plan_num';";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
			if ($sth->rows > 0) {
				($dest_zone,$zone_name) = $sth->fetchrow_array;
			}
		}

		$sth->finish();
	}

	if ($dest_zone == 0) {
		#    
		$RAD_REPLY{'h323-return-code'} = 'h323-return-code=1';
	}

	return ($dest_zone,$zone_name);
}

# source zone
# dest zone
# plan num
# default zone (national or international)
sub get_tariff($$$$) {
	my $src_zone = shift;
	my $dest_zone = shift;
	my $plan_num = shift;
	my $def_zone = shift;

	my $name_func = 'get_tariff';
	my $tariff = 0;
	my $time_diff = get_time_diff($src_zone);
	&radiusd::radlog($DEBUG,"pcp::$name_func() time diff = $time_diff");
	my $time_plan = get_time_plan(time() + ($time_diff * 3600));
	&radiusd::radlog($DEBUG,"pcp::$name_func() time paln = $time_plan");
	my $sql = "SELECT tariff FROM tariffs WHERE src_zone=$src_zone AND dest_zone=$dest_zone AND plan_num='$plan_num'";
	if (length($time_plan) == 1) {
		$sql .= " AND time_plan='$time_plan'";
	}
	$sql .= ";";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
			if ($sth->rows > 0) {
				($tariff) = $sth->fetchrow_array;
			}

			$sth->finish();
		}
	}

	if ($tariff == 0 && $def_zone ne '') {
		my $dz = "$src_zone\t$dest_zone\t$time_plan";
		if (defined($default_tariffs{$dz})) {
			&radiusd::radlog($DEBUG,"pcp::$name_func() default tariff $dz = ".$default_tariffs{$dz});
			return ($default_tariffs{$dz}, $time_plan);
		}
		if (defined($parameters{$def_zone})) {
			$dz = "$src_zone\t".$parameters{$def_zone}."\t$time_plan";
			if (defined($default_tariffs{$dz})) {
				&radiusd::radlog($DEBUG,"pcp::$name_func() default tariff $dz = ".$default_tariffs{$dz});
				return ($default_tariffs{$dz}, $time_plan);
			}
			$dz = "1\t".$parameters{$def_zone}."\t$time_plan";
			if (defined($default_tariffs{$dz})) {
				&radiusd::radlog($DEBUG,"pcp::$name_func() default tariff $dz = ".$default_tariffs{$dz});
				return ($default_tariffs{$dz}, $time_plan);
			}
		}
	}

	&radiusd::radlog($DEBUG,"pcp::$name_func() tariff = $tariff");
	return ($tariff, $time_plan);
}

# pin id
# conf id
sub get_inuse($$) {
	my $pin_id = shift;
	my $conf_id = shift;

	my $name_func = 'get_inuse';
	my %ret = ();
#	my %ret = ('code' => '', 'plan_num' => '', 'time_plan' => '',
#		'tariff' => 0, 'start_time' => '');

#	my $sql = "SELECT code,plan_num,time_plan,tariff FROM pin_in_use WHERE pin_id=$pin_id AND conf_id='$conf_id';";
	my $sql = "SELECT * FROM pin_in_use WHERE pin_id=$pin_id;";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
		if ($sth->rows > 0) {
			my $data = $sth->fetchrow_hashref;
			%ret = %{$data};
#		($ret{'code'},$ret{'plan_num'},$ret{'time_plan'},$ret{'tariff'},$ret{'start_time'}) = $sth->fetchrow_array;
		} else {
			&radiusd::radlog($DEBUG,"pcp::$name_func() ERROR: not found pin_id '$pin_id' in table pin_in_use");
		}
		}

		$sth->finish();
	}

	return %ret;
}

# plan num
sub get_called_data ($) {
	my $plan_num = shift;

	my %ret = ('dest_zone' => 0, 'zone_name' => '', 'number' => '', 
		    'tariff' => '', 'src_zone' => '', 'src_name' => '',
		    'def_zone' => '');

	#  cisco, ,   Called-Station-Id
	if (ref($RAD_REQUEST{'Called-Station-Id'}) eq 'ARRAY') {
		$ret{'number'} = $RAD_REQUEST{'Called-Station-Id'}[$#{ $RAD_REQUEST{'Called-Station-Id'} }];
	} else {
		$ret{'number'} = $RAD_REQUEST{'Called-Station-Id'};
	}
	($ret{'number'}, $ret{'def_zone'}) = cut_code_prefix($ret{'number'});
	my $max_length_code = get_max_length_code($ret{'number'}, $plan_num, $g_fields{$ret{'def_zone'}}, $g_tables{$ret{'def_zone'}});

	if ($max_length_code == 0) {
#&radiusd::radlog(0,"get_called_data() $pnumber,$c,$cc,$tariff,$plan_num max_length_code=$max_length_code");
		#    
		return %ret;
	}

	$ret{'code'} = substr($ret{'number'}, 0, $max_length_code);

	($ret{'dest_zone'}, $ret{'zone_name'}) = get_dest_zone($ret{'code'}, $plan_num, $g_fields{$ret{'def_zone'}}, $g_tables{$ret{'def_zone'}});

	if ($ret{'dest_zone'} == 0) {
		#    
		return %ret;
	}


	return %ret;
}

# pin_id
sub set_status_in_use($) {
	my $pin_id = shift;
	my $datetime = get_curtime();

#	if (check_pin_in_use($pin_id) == RLM_MODULE_OK) {
		my $sql = "UPDATE pin_in_use SET status='i',start_time='$datetime' where pin_id=$pin_id;";
		sql_do($sql, 'set_status_in_use()');
#	}
}

sub del_pin_in_use {
	my $pin_id = shift;
	my $status = shift || '';

&radiusd::radlog($DEBUG,"pcp::del_pin_in_use() DEBUG!!! DELETE FROM status = $status!");
	my $sql = "DELETE FROM pin_in_use WHERE pin_id=$pin_id";
	if ($status eq '') {
		$sql .= " and status is NULL";
	}
	$sql .= ";";
	sql_do($sql, 'del_pin_in_use');
}

sub get_time_plan($) {
	my $ctime = shift;

	my ($min,$hour,$mday,$mon,$wday) = (localtime($ctime))[1,2,3,4,6];
	my $count = 0;

	while ($count < scalar(@g_time_plan)) {
		my ($plan, $int) = split(/\t/, $g_time_plan[$count]);
		my @ints = split(/,/, $int);

		foreach my $i (@ints) {
			if ($i =~ /(\d+):(\d+)-(\d+):(\d+)/) {
				next if ($hour < $1 || $hour > $3);
				if ($hour > $1 || $hour < $3) {
					return $plan;
				}
				if ($hour == $1 && $min >= $2) {
					return $plan;
				}
				if ($hour == $3 && $min <= $4) {
					return $plan;
				}
			} else {
				if ($i =~ /(\w{3})\s+(\d+)/) {
					if (defined($mons{$1}) && $mons{$1} == $mon && $mday == $2) {
						return $plan;
					}
				} else {
					if (defined($wdays{$i}) && $wdays{$i} == $wday) {
						return $plan;
					}
				}
			}
		}

		$count ++;
	}

	return '';
}

# check block destination telephone number
# pin id
# number
sub check_blocked_dest ($$)
{
	my $pin_id = shift;
	my $pnumber = shift;

	my $name_func = 'check_blocked_dest';
	my $ret = 0;
	my $found = '';
&radiusd::radlog($DEBUG,"pcp::check_blocked_dest() blocked $pnumber");
	my $sql = "SELECT * FROM blocked WHERE pin_id IS NULL OR pin_id=$pin_id ORDER BY length(code) DESC;";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {

		while ((my $data = $sth->fetchrow_hashref)) {
&radiusd::radlog($DEBUG,"pcp::check_blocked_dest() blocked $pnumber\t".$data->{code});
			if ($pnumber =~ /^$data->{code}/) {
				$found = $data->{code};
				$ret = 1;
				last;
			}
		}
		}

		$sth->finish();
	}

	return $ret;
}

# check block AON telephone number
# Caller-Id
# plan num
# source
sub check_block_ani($$$) {
	my $ani = shift;
	my $plan_num = shift;
	my $source = shift;

	my $ret = 1;
	my $name_func = 'check_block_ani';

	if (length($ani) == 0) {
		return $ret;
	}

	my $code = substr($ani, 0, 1);
	my $sql = "SELECT * FROM block WHERE client_id like '$code\%' AND plan_num='$plan_num' AND source_id=$source;";
	my $sth = sql_prepare($sql, $name_func);
	if ($sth) {
		if (sql_execute($sth, $name_func)) {
			my $len = 0;
			while (my $data = $sth->fetchrow_hashref()) {
				my $l = length($data->{'client_id'});
				if ($len < $l && $ani =~ /^$data->{'client_id'}/) {
					$len = $l;
					$ret = $data->{'color'};
					$code = $data->{'client_id'};
				}
			}
			if ($ret == 0) {
				&radiusd::radlog($DEBUG,"pcp::$name_func() CLI $ani for plan num '$plan_num' and source id $source blocked code $code");
			}
		}
		$sth->finish();
	}

	return $ret;
}

sub get_curtime {
	my @times = localtime();
	$times[5] += 1900;
	$times[4] ++;
	return sprintf("%s-%02d-%02d %02d:%02d:%02d", $times[5],$times[4],$times[3],$times[2],$times[1],$times[0]);
}

sub get_called_st_id($)
{
    my $ret = '';
#&radiusd::radlog(0,"pcp::get_called_st_id() " . ref($_[0]));

    if (ref($_[0]) eq 'ARRAY') {
#	my @arr = $_[0];
#	$ret = $arr[$#arr];
	$ret = $_[0]->[scalar($_[0]) - 1]
&radiusd::radlog($DEBUG,"pcp::get_called_st_id() $ret!!!!!!!!!!");
    } else {
	$ret = $_[0];
    }

    return $ret;
}

# query
# name func
sub sql_do($$)
{
	my $sql = shift;
	my $name_func = shift || '';

	&radiusd::radlog($DEBUG,"pcp::$name_func() $sql") if ($g_log);
	if (!$dbh->do($sql)) {
		&radiusd::radlog(0,"pcp::$name_func() Error do query $sql :" . $dbh->err);
		return 0;
	}

	return 1;
}

# query
# name func
sub sql_prepare($$)
{
	my $sql = shift;
	my $name_func = shift || '';

	&radiusd::radlog($DEBUG,"pcp::$name_func() $sql") if ($g_log);
	my $sth = $dbh->prepare($sql);
	if (!$sth) {
		&radiusd::radlog(0,"pcp::$name_func() Error prepare query $sql: " . $dbh->err);
	}

	return $sth;
}

# sth
# name func
sub sql_execute($$)
{
	my $sth = shift;
	my $name_func = shift || '';

	if (!$sth->execute()) {
		&radiusd::radlog(0,"pcp::$name_func() Error execute query: " . $sth->err);
		if ($sth->err == 2006) {
			Init_db();
		}
		return 0;
	}

	return 1;
}

# name file
sub load_config($)
{
	my $filename = shift;

	my %options = ();

	if (open(CONF, "< $filename")) {
		while (<CONF>) {
			if (/^\n/) {
			} elsif (/^\#/) {
			} else {
				$_ =~ s/\n//g;
				my @a = split(/=/);
				my $c = 2;
				while ($c < scalar(@a)) {
					$a[1] .= $a[$c];
					$c++;
				}
				if (scalar(@a) > 1) {
					$a[0] =~ s/^\s+//g;
					$a[0] =~ s/\s+$//g;
					$a[1] =~ s/^\s+//g;
					$a[1] =~ s/\s+$//g;
					$options{$a[0]} = $a[1];
				}
			}
		}
		close(CONF);
		
	} else {
		&radiusd::radlog(0,"pcp::load_config() Error open config file " . $filename);
	}

	return %options;
}

#BEGIN {
#	$g_log = 0;
#	$dbh_connect = 0;
#	$dbh = '';
#	&Init_db();
#	&Init_gdata();
#	$dbh->disconnect() if ($dbh);
#	$dbh_connect = 0;
#}

#END {
#	if ($dbh && $dbh_connect) {
#		$dbh->disconnect();
#		$dbh_connect = 0;
#	}
#}
